import { createContext } from "react";

export type FormFieldValue = string;

export type FormsContextT = {
  formsById: {
    [id: string]: {
      fieldsByName: {
        [name: string]: FormFieldValue;
      };
    };
  };
  // API - Internal Functions
  _mountForm(id: string): void;
  _getFormValues<
    V extends { [name: string]: FormFieldValue } = {
      [name: string]: FormFieldValue;
    },
  >(
    formId: string,
  ): V | null;
  _getFormFieldValue<V extends FormFieldValue = FormFieldValue>(
    formId: string,
    fieldName: string,
  ): V | null;
  _setFormFieldValue(
    formId: string,
    fieldName: string,
    fieldValue: FormFieldValue,
  ): void;
  _unmountForm(id: string): void;
};

const DEFAULT_FORMS_CONTEXT_VALUE: FormsContextT = {
  formsById: {},
  _mountForm() {},
  _getFormValues<V>() {
    return {} as V;
  },
  _getFormFieldValue() {
    return null;
  },
  _setFormFieldValue() {},
  _unmountForm() {},
};

export const FormsContext = createContext<FormsContextT>(
  DEFAULT_FORMS_CONTEXT_VALUE,
);
