import React, { FC, useMemo, useState, useCallback } from "react";

import { FormFieldValue, FormsContext, FormsContextT } from "./FormsContext";

export const FormsProvider: FC = ({ children }) => {
  const [formsById, setFormsById] = useState<FormsContextT["formsById"]>({});

  const _mountForm = useCallback(
    (id: string) => {
      setFormsById((prev) => ({
        ...prev,
        [id]: {
          fieldsByName: {},
        },
      }));
    },
    [setFormsById],
  );

  const _getFormValues = useCallback(
    <
      V extends { [name: string]: FormFieldValue } = {
        [name: string]: FormFieldValue;
      },
    >(
      formId: string,
    ): V | null => {
      return (formsById?.[formId].fieldsByName as V) || null;
    },
    [formsById],
  );

  const _getFormFieldValue = useCallback(
    <V extends FormFieldValue = FormFieldValue>(
      formId: string,
      fieldName: string,
    ): V | null => {
      return (formsById?.[formId]?.fieldsByName?.[fieldName] as V) || null;
    },
    [formsById],
  );

  const _setFormFieldValue = useCallback(
    (formId: string, fieldName: string, fieldValue: FormFieldValue) => {
      setFormsById((prev) => ({
        ...(prev || {}),
        [formId]: {
          ...(prev?.[formId] || {}),
          fieldsByName: {
            ...(prev?.[formId]?.fieldsByName || {}),
            [fieldName]: fieldValue,
          },
        },
      }));
    },
    [setFormsById],
  );

  const _unmountForm = useCallback(
    (id: string) => {
      setFormsById((prev) => ({
        ...Object.entries(prev)
          .filter(([formId]) => formId !== id)
          .reduce((acc, [formId, form]) => {
            acc = {
              ...acc,
              [formId]: form,
            };
            return acc;
          }, {} as FormsContextT["formsById"]),
      }));
    },
    [setFormsById],
  );

  const ctxValue = useMemo(
    () => ({
      _mountForm,
      _getFormValues,
      _getFormFieldValue,
      _setFormFieldValue,
      _unmountForm,
      formsById,
    }),
    [
      _mountForm,
      _getFormValues,
      _getFormFieldValue,
      _setFormFieldValue,
      _unmountForm,
      formsById,
    ],
  );
  return (
    <FormsContext.Provider value={ctxValue}>{children}</FormsContext.Provider>
  );
};
